{% load i18n %}
<div class="alert alert-danger alert-dismissible" role="alert">
    <button type="button" class="btn-close" data-bs-dismiss="alert" aria-label="{% trans 'close' %}"></button>
    {% for error in errors %}
        {{ error }}{% if not forloop.last %}<br>{% endif %}
    {% endfor %}
</div>
